require 'digest/sha2'

module Sinatra
    module Authentication
        module Cookies
            module Hashing
                extend self

                def encrypt(user_id, ip, key, salt = self.generate_salt)
                    hash = serialize(hash(user_id, ip, key, salt), salt)
                    { :hash => hash, :salt => salt }
                end

                def decrypt(crypted, key)
                    Encryptor.decrypt(crypted, :key => key)
                end

                def check?(user_id, ip, key, crypted)
                    hash, salt = unserialize(crypted)
                    self.hash(user_id, ip, key, salt) == hash
                end

            protected
                def generate_salt
                    salt = ""
                    64.times { salt << (i = Kernel.rand(62); i += ((i < 10) ? 48 : ((i < 36) ? 55 : 61))).chr }
                    salt
                end

                # This method can be customized
                def hash(user_id, ip, key, salt)
                    Digest::SHA512.hexdigest("#{ user_id }:#{ ip }:#{ key }:#{ salt }")
                end

                def serialize(hash, salt)
                    hash + salt
                end

                def unserialize(serialized)
                    return serialized[0..127], serialized[128..-1]
                end
            end
        end
    end
end
